@@ -1,8 +1,7 @@
 from __future__ import annotations
 
 import json
-import sys
-from functools import lru_cache
+from functools import cached_property, lru_cache
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
@@ -10,6 +9,7 @@ from typing import (
     Dict,
     Iterator,
     List,
+    Literal,
     NamedTuple,
     Optional,
     Tuple,
@@ -17,19 +17,10 @@ from typing import (
     Union,
 )
 
-import rich.repr
-
-if sys.version_info >= (3, 8):
-    from functools import cached_property
-else:
-    from cartes.utils.cache import cached_property
-
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
-
 import geopandas as gpd
+import rich.repr
+from ipyleaflet import Map as LeafletMap
+from ipyleaflet import Polygon as LeafletPolygon
 
 import numpy as np
 import pyproj
@@ -170,6 +161,56 @@ class Airspace(ShapelyMixin):
     def shape(self) -> "BaseGeometry":
         return self.flatten()
 
+    def leaflet(self, **kwargs: Any) -> LeafletPolygon:
+        """Returns a Leaflet layer to be directly added to a Map.
+
+        The elements passed as kwargs as passed as is to the Polygon
+        constructor.
+        """
+
+        shape = self.flatten()
+
+        kwargs = {**dict(weight=3), **kwargs}
+        coords: List[Any] = []
+
+        def unfold(shape: Polygon) -> Iterator[Any]:
+            yield shape.exterior
+            yield from shape.interiors
+
+        if shape.geom_type == "Polygon":
+            coords = list(
+                list((lat, lon) for (lon, lat, *_) in x.coords)
+                for x in unfold(shape)
+            )
+        else:
+            coords = list(
+                list((lat, lon) for (lon, lat, *_) in x.coords)
+                for piece in shape.geoms
+                for x in unfold(piece)
+            )
+
+        return LeafletPolygon(locations=coords, **kwargs)
+
+    def map_leaflet(
+        self,
+        *,
+        zoom: int = 6,
+        **kwargs: Any,
+    ) -> Optional[LeafletMap]:
+        from ipywidgets import HTML
+
+        if "center" not in kwargs:
+            (lon, lat), *_ = self.shape.centroid.coords
+            kwargs["center"] = (lat, lon)
+
+        m = LeafletMap(zoom=zoom, **kwargs)
+
+        elt = m.add_layer(self)
+        elt.popup = HTML()
+        elt.popup.value = self.designator
+
+        return m
+
     def __getitem__(self, *args: int) -> ExtrudedPolygon:
         return self.elements.__getitem__(*args)
 
@@ -446,7 +487,7 @@ def _flight_intersects(
             assert layer.lower is not None
             assert layer.upper is not None
             if isinstance(ix, base.BaseMultipartGeometry):
-                for part in ix:
+                for part in ix.geoms:
                     if any(
                         100 * layer.lower < x[2] < 100 * layer.upper
                         for x in part.coords
