@@ -8,7 +8,6 @@ from typing import (
     Type,
     TypeVar,
     Union,
-    cast,
 )
 
 from pyModeS import adsb
@@ -83,8 +82,6 @@ class RawData(DataFrameMixin):
 
             progressbar = identity
 
-        progressbar = cast(ProgressbarType[U], progressbar)
-
         data = self.data.rename(  # fill with other common renaming rules
             columns={
                 "mintime": "timestamp",
@@ -120,7 +117,7 @@ class RawData(DataFrameMixin):
 
     def assign_type(self) -> "RawData":
         def get_typecode(msg: Union[bytes, str]) -> Optional[int]:
-            tc = adsb.typecode(msg)
+            tc = adsb.typecode(msg)  # type: ignore
             if 9 <= tc <= 18:
                 return 3
             elif tc == 19:
