@@ -11,6 +11,8 @@ from shapely.geometry.base import BaseGeometry
 from ...core.mixins import GeoDBMixin
 from ...core.structure import Navaid, Route
 
+_log = logging.getLogger(__name__)
+
 __github_url = "https://raw.githubusercontent.com/"
 base_url = __github_url + "xoolive/traffic/master/data/navdata"
 
@@ -113,7 +115,7 @@ class Airways(GeoDBMixin):
         if not (self.cache_dir / "traffic_airways.pkl").exists():
             self.download_data()
         else:
-            logging.info("Loading airways database")
+            _log.info("Loading airways database")
             self._data = pd.read_pickle(self.cache_dir / "traffic_airways.pkl")
 
         if self._data is not None:
