@@ -1,9 +1,9 @@
-from collections import defaultdict
 from pathlib import Path
+from typing import DefaultDict
 
 
-class RequestsDict(defaultdict):
-    def __missing__(self, key):
+class RequestsDict(DefaultDict[str, str]):
+    def __missing__(self, key: str) -> str:
         candidate_file = Path(__file__).parent / (key + ".xml")
         if not candidate_file.exists():
             raise KeyError(key)
