@@ -11,6 +11,8 @@ from shapely.ops import orient
 from ...core.airspace import Airspace, Airspaces, ExtrudedPolygon
 from . import ADDS_FAA_OpenData
 
+_log = logging.getLogger(__name__)
+
 
 class Airspace_Boundary(ADDS_FAA_OpenData, Airspaces):
 
@@ -68,7 +70,7 @@ class Airspace_Boundary(ADDS_FAA_OpenData, Airspaces):
             )
 
             if not airspace.shape.is_valid:
-                logging.warning(f"Invalid shape part {name}, skipping...")
+                _log.warning(f"Invalid shape part {name}, skipping...")
                 continue
 
             if name in airspaces:
